@@ -2,16 +2,18 @@
 This module extends the flowchart module in kordesii.utils in order to add ProcessorContext
 tracking and emulation.
 """
-
+from __future__ import annotations
 import logging
 import warnings
 from copy import deepcopy
-from typing import Optional
+from typing import TYPE_CHECKING, Optional
 
 import idautils
 
 from kordesii.utils import flowchart
-from .cpu_context import ProcessorContext
+
+if TYPE_CHECKING:
+    from .cpu_context import ProcessorContext
 
 
 logger = logging.getLogger(__name__)
@@ -25,24 +27,29 @@ class PathNode(flowchart.PathNode):
 
     _cache = {}
 
-    def __init__(self, bb: "BasicBlock", prev: Optional["PathNode"]):
+    def __init__(self, bb: BasicBlock, prev: Optional[PathNode]):
         """
         Initialize a path node.
 
         :param bb: The underlying basic block for this node.
         :param prev: The parent node that points to this node.
         """
-        super(PathNode, self).__init__(bb, prev)
+        super().__init__(bb, prev)
         # TODO: Add caching for multiple init_contexts?
         self._context = None
         self._context_ea = None  # ea that the context has been filled to (but not including)
         self._init_context = None  # the context used at the starting path
+        self._call_depth = 0       # the number of calls deep we are allowed to emulate
 
-    def cpu_context(self, ea=None, *, init_context: ProcessorContext):
+    def cpu_context(self, ea=None, *, call_depth: int = 0, init_context: ProcessorContext) -> ProcessorContext:
         """
         Returns the cpu context filled to (but not including) the specified ea.
 
         :param int ea: address of interest (defaults to the last ea of the block)
+        :param call_depth: Number of function calls we are allowed to emulate into.
+            When we hit our limit (depth is 0), emulation will no longer jump into function calls.
+            (Defaults to not emulating into any function calls.)
+            NOTE: This does not affect call hooks.
         :param init_context: Initial context to use for the start of the path. (required)
 
         :return cpu_context.ProcessorContext: cpu context
@@ -59,9 +66,11 @@ class PathNode(flowchart.PathNode):
         else:
             end = ea
 
-        # Determine if we need to force the creation of a new context if we have a different init_context.
-        new_init_context = self._init_context != init_context
+        # Determine if we need to force the creation of a new context if we have a different init_context
+        # or call_depth.
+        new_init_context = self._init_context != init_context or self._call_depth != call_depth
         self._init_context = init_context
+        self._call_depth = call_depth
 
         assert end is not None
         # Fill context up to requested endpoint.
@@ -69,11 +78,11 @@ class PathNode(flowchart.PathNode):
             # Create context if:
             #   - not created
             #   - current context goes past requested ea
-            #   - given init_context is different from the previously given init_context.
+            #   - given init_context/call_depth is different from the previously given init_context/call_depth.
             if not self._context or self._context_ea > end or new_init_context:
                 # Need to check if there is a prev, if not, then we need to create a default context here...
                 if self.prev:
-                    self._context = self.prev.cpu_context(init_context=init_context)
+                    self._context = self.prev.cpu_context(call_depth=call_depth, init_context=init_context)
                     # Modify the context for the current branch if required
                     self._context.prep_for_branch(self.bb.start_ea)
                 else:
@@ -85,7 +94,7 @@ class PathNode(flowchart.PathNode):
                 # Fill context up to requested ea.
                 logger.debug("Emulating instructions 0x%08X -> 0x%08X", self._context_ea, end)
                 for ip in idautils.Heads(self._context_ea, end):
-                    self._context.execute(ip)
+                    self._context.execute(ip, call_depth=call_depth)
 
             self._context_ea = end
 
